#pylint: disable=F0401,C0103,C0301,R0903,E1101

"""
This is the client-side part of the SRP login protocol.
"""

import requests
import json
import base64

# because the client and server are running on the same computer (right now),
# the lib directory is located elsewhere, and we have to hack $PATH to find it
import sys
sys.path.append('../lib')
from pwgen_lib import Password

import srp
import srp_patches
srp_patches.patch() # monkeypatch!

class LoginClient(object):
    """Store everything in a wrapper class for convenience"""

    def __init__(self, username, password_txt, endpoint):
        """SRP begins as soon as the class is constructed"""

        self.username = username
        self.endpoint = endpoint
        self.session = requests.Session() # use a session so we have a persisting cookiejar

        resp = self._post(1, dict(username=self.username))

        password = Password(txt=password_txt)
        # we use the p from our protocl as the N for SRP, and the y from our
        # protocol as the g for SRP -- as we note in our protocl paper, this
        # weakens SRP slightly but we believe that it's still secure
        usr = srp.User(self.username, password.num, ng_type=srp.NG_CUSTOM, n_hex=hex(resp['p']), g_hex=hex(resp['y']))
        _, A = usr.start_authentication()
        # we need to base64 encode these values because otherwise they contain
        # characters we can't handle
        resp = self._post(2, dict(A=base64.b64encode(A)))

        s = base64.b64decode(resp['s'])
        B = base64.b64decode(resp['B'])
        M = usr.process_challenge(s, B)
        if M is None:
            return
        resp = self._post(3, dict(M=base64.b64encode(M)))

        HAMK = base64.b64decode(resp['HAMK'])
        usr.verify_session(HAMK)
        if not usr.authenticated():
            return
        self.authentication_token = resp['token']

    def _post(self, step, payload):
        """Helper method to post data to the login/authentication service"""

        headers = {'content-type': 'application/json'}
        payload['username'] = self.username
        url = "%s/%s" % (self.endpoint, step)

        r = self.session.post(url, data=json.dumps(payload), headers=headers)
        if r.status_code != requests.codes.ok:
            raise Exception
        return r.json()

    def get_token(self):
        """Return the user's authentication token"""
        return self.authentication_token
